﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Otter;
using TiledSharp;
using SpaceMonsterRepresent.Entities.humans;
using SpaceMonsterRepresent.Util;
using SpaceMonsterRepresent.Entities.objects;
using SpaceMonsterRepresent.UI;

namespace SpaceMonsterRepresent.Scenes
{
    class Cave : Scene
    {
        public const int _width = 720 * 3;
        public const int _height = 480 * 2;

        public Tilemap tilemap = null;
        public GridCollider grid = null;

        public Cave()
        {
            tilemap = new Tilemap(Assets.CAVETILESET, _width, _height, Global.GRID_WIDTH, Global.GRID_HEIGHT);
            grid = new GridCollider(_width, _height, Global.GRID_WIDTH, Global.GRID_HEIGHT);
            Entity gridEntity = new Entity(0, 0, null, grid);
            Add(gridEntity);

            string mapToLoad = Assets.CAVE;
            LoadWorld(mapToLoad);
            AddGraphic(tilemap);

            Global.cam = new Camera(100, 2);
            Global.player = new Entities.Player(670, 200);
            Add(new Pedistal(0, 0));
            Add(new Beam(0, 0));
            Add(Global.player);

            Add(new UIGrid(0, 0));
            Add(new Icons(0, 0));
            Add(new Life(0, 0));
            Add(new RedShirtNumber());
            Add(new HealthNumber());
            Add(new Score());

            //Add(Global.camShaker);
            Add(Global.cam);
        }
        public override void Begin()
        {
            base.Begin();
            Global.cam.adjustToPlayer(Global.GAME.Scene.Width, Global.GAME.Scene.Height, Global.PlayerX, Global.PlayerY);
        }
        public override void Update()
        {
            base.Update();
            Global.cam.followPlayer(Global.GAME.Scene.Width, Global.GAME.Scene.Height, Global.PlayerX + 100, Global.PlayerY + 100);
            if (Global.PlayerX >= Global.GAME.Scene.Width || Global.PlayerY >= Global.GAME.Scene.Width)
            {
                Global.GAME.RemoveScene();
                Global.GAME.AddScene(new Forest());
                CenterCamera(Global.PlayerX, Global.PlayerY);
            }
        }
        private void LoadWorld(string map)
        {
            TmxMap _map = new TmxMap(map);

            for (int x = 0; x < _map.Width; x++)
            {
                for (int y = 0; y < _map.Height; y++)
                {
                    int i = y * _map.Width + x;
                    tilemap.SetTile(x, y, _map.Layers[0].Tiles[i].Gid - 1);
                }
            }
        }
    }
}