﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Otter;
using TiledSharp;
using SpaceMonsterRepresent.Entities.humans;
using SpaceMonsterRepresent.Util;
using SpaceMonsterRepresent.Entities.objects;
using SpaceMonsterRepresent.UI;

namespace SpaceMonsterRepresent.Scenes
{
    class Forest : Scene
    {
        public const int _width = Global.MW * 3;
        public const int _height = Global.MH * 2;

        public Tilemap tilemap = null;
        public GridCollider grid = null;

        public Forest()
        {
            tilemap = new Tilemap(Assets.TILESET, _width, _height, Global.GRID_WIDTH, Global.GRID_HEIGHT);
            grid = new GridCollider(_width, _height, Global.GRID_WIDTH, Global.GRID_HEIGHT);
            Entity gridEntity = new Entity(0, 0, null, grid);
            Add(gridEntity);

            //UseCameraBounds = true;

            //var mapGrid = new Grid(1000, 1000, 20, 20, Color.Grey);
            //AddGraphic(mapGrid);

            string mapToLoad = Assets.FOREST;
            LoadWorld(mapToLoad);
            AddGraphic(tilemap);

            Global.cam = new Camera(100, 2);
            Add(new Mountain(40, -10));
            Global.player = new Entities.Player(100, 100);
            for (int i = 0; i <= 1180; i++)
            {
                Add(new ShrubColor(Otter.Rand.Int(-140, Global.MW - 16), Otter.Rand.Int(-140, Global.MH - 16)));
            }
            for (int i = 0; i <= 1180; i++)
            {
                Add(new FlowerBush(Otter.Rand.Int(-140, Global.MW - 16), Otter.Rand.Int(-140, Global.MH - 16)));
            }
            Add(new RedShirt(540, 340));
            AddSoldiers();
            for (int i = 0; i <= 200; i++)
            {
                Add(new RedShirt(Otter.Rand.Int(0, Global.MW), Otter.Rand.Int(0, Global.MH)));
            }
            for (int i = 0; i <= 1180; i++)
            {
                Add(new Shrub(Otter.Rand.Int(-140, Global.MW - 16), Otter.Rand.Int(-140, Global.MH - 16)));
            }
            Add(Global.player);
            Add(new Tree(400, 400));
            for (int i = 0; i <= 1180; i++)
            {
                Add(new Shrub(Otter.Rand.Int(-140, Global.MW - 16), Otter.Rand.Int(-140, Global.MH - 16)));
            }
            for (int i = 0; i <= 1180; i++)
            {
                Add(new Bush(Otter.Rand.Int(-140, Global.MW - 16), Otter.Rand.Int(-140, Global.MH - 16)));
            }
            //testing remove this comment when ready for deployment?
            for (int i = 0; i <= 80; i++)
            {
                Add(new Tree(Otter.Rand.Int(-140, Global.MW-16), Otter.Rand.Int(-140, Global.MH-16)));
            }
            Add(new UIGrid(0, 0));
            Add(new Icons(0, 0));
            Add(new Life(0, 0));
            Add(new RedShirtNumber());
            Add(new HealthNumber());
            Add(new Score());

            //Add(Global.camShaker);
            Add(Global.cam);
        }
        public override void Begin()
        {
            base.Begin();
            Global.cam.adjustToPlayer(Global.MW, Global.MH, Global.PlayerX, Global.PlayerY);
        }
        public override void Update()
        {
            base.Update();
            Global.cam.followPlayer(Global.MW, Global.MH, Global.PlayerX + 100, Global.PlayerY + 100);
        }
        private void LoadWorld(string map)
        {
            TmxMap _map = new TmxMap(map);

            for(int x = 0; x < _map.Width; x++)
            {
                for (int y = 0; y < _map.Height; y++)
                {
                    int i = y * _map.Width + x;
                    tilemap.SetTile(x, y, _map.Layers[0].Tiles[i].Gid - 1);
                }
            }
        }
        private void AddSoldiers()
        {
            //pick a point on the map, add a random cluster on the map? oh i don't know. they are supposed to spawn in a huge group. that will probably just have to happen that way.
            var randX = 350;//Otter.Rand.Int(0, Global.MW);
            var randY = 350;//Otter.Rand.Int(0, Global.MH);
            for (int i = 0; i <= 50; i++){
                Add(new Soldier(randX, randY));
            }
        }
    }
}
